% January 2015, F. DE FIORE and H. UHLIG. "Corporate debt structure and the financial crisis", JMCB 2015 and ECB WP No. 1759.
%This file computes the steady state of the model presented in the paper

%The steady state is calibrated such that 
%A) the model replicates the facts on the EA, documented in the paper "Bank Finance vs Bond Finance" (JMCB, 2011) by F. De Fiore and H. Uhlig. 
   %In particular,the free parameters (s_aggr, chi, tau, sigma_eps1,sigma_eps2, sigma_eps3) are set to replicate the following facts: 
   %1) ratio of bank finance to bond finance = 5.48
   %2) aggr_debt_to_equity =.64
   %3) risk_premium_cmf = .0036 
   %4) risk_premium_bank = .0030 
   %5) cond_monitored_if_cmf = .0124 
   %6) expected_return/.023  
%B) replicates the Euro area average annual inflation rate equal to 2.01 percent over the period 1999-2009.

%Remember to assign the appropriate value to DOING_ESTIMATION
%DOING_ESTIMATION=0 solves the steadystate using the parameter valuesassigned below; 
%DOING_ESTIMATION=1 solves it in connection with the search for parameter values set up in estimation_us.m, using 
%the values chosen by the minimization routine 

if ~exist('DOING_SEARCH'),
    DOING_SEARCH = 0;
end;

if ~exist('DOING_ESTIMATION'),
    DOING_ESTIMATION = 0;
end;

if ~exist('TABULATE_RESULTS'),
    TABULATE_RESULTS = 0;
end;

if ~exist('DO_PERMANENT_SHOCK'),
    DO_PERMANENT_SHOCK = 0;
end;

if ~exist('DO_NO_CMF_IN_CHOICE'),
    DO_NO_CMF_IN_CHOICE = 0;
end;

if (~DOING_ESTIMATION) & (~TABULATE_RESULTS) & (~DO_PERMANENT_SHOCK),
    % disp('Hello!');
    if (~DOING_SEARCH) & (~DO_NO_CMF_IN_CHOICE),
        clear all
        DOING_SEARCH=0;
    end;
    DOING_ESTIMATION = 0;
    TABULATE_RESULTS = 0;
    DO_PERMANENT_SHOCK = 0;
    
end;


%if (~DOING_ESTIMATION) & (~DOING_SEARCH),
if (~DOING_SEARCH),
    global s_aggr beta sa sb sbp sc phim phif ms m chi kai xi mon r delta rhsgrd2 omgrd2 s_ng...
        eps2_weight eps2_grd eps2_quad_grid s_bank s_cmf om_cmf omsolve_bank...
        sigma_eps1 sigma_eps2 sigma_eps3 tau eta kappa le profits_cmf...
        lb rp_b aver_default l A s_times_eps1_stp ya_int...
        s_times_eps1_grd eps2_cutoff undef_cmf undef_bank...
        cond_monitored_if_banking cond_monitored_if_cmf...
        frac_bank_continue_withchoice eps2_stp eps2_grd eps2_weight...
        prob_of_rect_eps2 y ya_y c_y ce_y I_y z_K n_y aggr_debt_equity...
        y_us facts DO_STANDARD rhsgrd2 omgrd2 frac_bank_dropout...
        sgrd_bank sgrd_cmf pi_bar Rnom TFP rp_bank ms d readme shock_counter...
        DO_NO_CMF_IN_CHOICE DO_NO_BANK_IN_CHOICE DOING_SEARCH eps2_cutoff_SS
end;

%%%%%%%%%%PARAMETERS%%%%%%%%%%%%%

%These parameters are set based on previous studies
beta=.99;
delta=.02;
alpha=.64;
rho=.95;
mon = 0.15;
zeta = 1; %1.01;
g_z=0;
varkappa = 1;
kappa = 3; %.1; Frisch elasticity of labor supply

% if ~exist('pi_bar'),
    pi_bar = 1.005; % beta; %1.006; %1.020; %*** average annual inflation rate equal to 2.01 percent, the observed average over the period 1999-2009.
% end;
TFP = 1.0; %*** probably just some arbitrary normalization...


format long

if (~DOING_ESTIMATION),
    
    if (~DOING_SEARCH),
        disp('Calibration quarterly:');
        disp('beta=.99, delta=.02, infl=.006');
        disp('targets: lb=5.48; aggr_debt_to_equity =.64; risk_premium_cmf=.0036; risk_premium_bank=.0030; cond_monitored_if_cmf = .0124; expected_return = .023');
  
    p = [1.000530592518364  0.016567805155764  0.022522394048695  0.171148598570017  3.195271417969750  0.009895619839787]; %mse=.07367
      
    end; 
    
    s_aggr = p(1);
    if exist('q'),
        s_aggr = q;
    end;
    chi = 0;
    sigma_eps1 = p(2);
    sigma_eps2 = p(3);
    sigma_eps3 = p(4);
    xi = p(5);
    tau = p(6); %p(6);
    
    if DO_PERMANENT_SHOCK,
        sigma_choice = 3;
        switch sigma_choice
            case 1
                sigma_eps1 = 1.1*p(2);
                sigma_eps2 = p(3);
                sigma_eps3 = p(4);
                readme = sprintf('%s','sigma_eps1 = 1.1*p(2)');
            case 2
                sigma_eps1 = p(2);
                sigma_eps2 = 1.1*p(3);
                sigma_eps3 = p(4);
                tau = p(6);
                readme = sprintf('%s','sigma_eps2 = 1.1*p(3)');
            case 3
                sigma_eps1 = p(2);
                sigma_eps2 = p(3);
                sigma_eps3 = 1.1*p(4);
                tau = p(6);
                readme = sprintf('%s','sigma_eps3 = 1.1*p(4)');
            case 4
                sigma_eps1 = p(2);
                sigma_eps2 = p(3);
                sigma_eps3 = p(4);
                tau = 1.1*p(6);%1.5*p(6);
                readme = sprintf('%s','tau = 1.1*p(6)');
            case 5
                sigma_eps1 = p(2);
                sigma_eps2 = p(3);
                sigma_eps3 = 1.05*p(4);
                tau = 1.03*p(6);
                varkappa = .97;
                readme = sprintf('%s','sigma_eps3 = 1.05*p(4), varkappa = .97 and tau = 1.03*p(6)');
            otherwise
                sigma_eps1 = 1.1*p(2);
                sigma_eps2 = 1.1*p(3);
                sigma_eps3 = 1.1*p(4);
                tau = 1.1*p(6);
                readme = sprintf('%s','sigma_eps1 = 1.1*p(2), sigma_eps2 = 1.1*p(3), sigma_eps3 = 1.1*p(4), tau = 1.1*p(6)');
        end;
    end;
    
    
end;

if ~DOING_SEARCH,
    disp('EA model')
    disp(sprintf('s-aggr = %5.15f',s_aggr))
    disp(sprintf('sigma_eps1 = %5.15f',sigma_eps1))
    disp(sprintf('sigma_eps2 = %5.15f',sigma_eps2))
    disp(sprintf('sigma_eps3 = %5.15f',sigma_eps3))
    disp(sprintf('xi = %5.15f',xi))
    disp(sprintf('tau = %5.15f',tau))
    
end;

%*** Compute the nominal return
Rnom = 1/beta * pi_bar;

%Compute the rental rate for capital
r = (1/beta-1+delta);

%Compute shares, for a given s_aggr
choice;

eps2_cutoff_SS = eps2_cutoff;
sa = share_abstain;
sb = share_bank;
sbp = share_bank*(1-cond_dropout_if_banking);
sc = share_cmf;
soa = sa + sb-sbp;

%Calculate risk premia
integr_over_eps1_vec;

%%compute risk premia (as defined in rateb_vec.m)
risk_premium_bank =  sum( (1-undef_bank).*res.*(1-res2).*rp_bank.*prob_of_rectangle )/sbp;
om_cmf_interp = interp1([s_bot-1,sgrd_cmf,s_top+1],[0,om_cmf,0],s_ng);
rp_cmf = (1/Rnom)*(xi/(xi-1))*TFP*om_cmf_interp.*s_ng - 1;
risk_premium_cmf =  sum( (1-undef_cmf).*res2.*res3.*rp_cmf.*prob_of_rectangle )/sc ;
risk_premium_diff = risk_premium_cmf - risk_premium_bank;

%Compute the real wage
w = alpha*(s_aggr/TFP)^(-1/alpha)*((1-alpha)/r)^((1-alpha)/alpha);

% Normalize aggregate consumption, and solve for eta
c = 1;

%Compute ratio of project size to initial capital x_z
x_z =  xi*(1-delta+r)*((1-tau)*sbp+sc);

%Compute ratio of capital to project size, K_x
K_x = (1-alpha)/r;

%Compute ratio of labor to project size, l_x
l_x = alpha/w;

%Compute ratio k_z
k_z=K_x*x_z-1;

%Compute z
c_z = (Rnom-1)*((1-tau)*sbp+sc)*(xi-1)*(1-delta+r) + (w*l_x*x_z+(r-delta)*k_z);
z = c/c_z;

%Compute aggregate project size
x = z*x_z;

%Compute aggregate capital K
K = x*K_x;

%Compute household capital k
k = k_z*z;

%Compute labor l
l = l_x*x;

% Compute eta
eta = w/l^(1/kappa);

%Compute aggregate investment
I=delta*K;

%Compute net worth
n=(1-delta+r)*z;

%Compute aggregate output
y_mat = sum( ((1/s_aggr)*(s_times_eps1_grd'*eps2_grd).*res_o.*(ones(le,1)*prob_of_rect_eps2))');
y_mat_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,y_mat,0],s_ng);
phiy = (1-tau)*sum((1-undef_cmf).*res.*(1-res2).*y_mat_interp.*prob_of_rectangle)...
    + sum((1/s_aggr)*(1-undef_cmf).*res2.*res3.*s_ng.*prob_of_rectangle);
y = phiy*s_aggr*xi*n;

%Compute exogenous govt expenditure
g=g_z*z;
g_y=g/y;

% Compute investment-output ratio
I_y = I/y;

%Compute aggregate entrepreneurial consumption
varthet_integr_over_eps1_vec;
phif_check = share_abstain...
    + sum((1-tau)*((1-undef_bank).*res.*(1-res2).*varthet_a+frac_bank_dropout_withchoice).*prob_of_rectangle)...
    + xi*sum((1-undef_cmf).*res2.*res3.*s_ng.*f_func(om_cmf_interp,sigma_cmf).*prob_of_rectangle);
om_cmf_interp= interp1(rhsgrd2,omgrd2,s_ng);
Fc_interp = xi*s_ng.*f_func(om_cmf_interp,sigma_cmf)-chi*xi;
Fb_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0, profits_bank,0],s_ng);
phif = share_abstain...
    + sum((1-undef_cmf).*res.*(1-res2).*Fb_interp.*prob_of_rectangle)...
    + sum((1-undef_cmf).*res2.*res3.*Fc_interp.*prob_of_rectangle);

%Compute gamma, e and expected return to z (with death probability)
y_e = phif*n;
e = y_e - z;
gamma = 1 - e./(phif*varkappa*n);
expected_return = phif*(1-delta+r)-1;
C_y = (c+e)/y;

%Compute monetary transfers from the central bank
d = ((1-tau)*sbp+sc)*(xi-1)*(1-delta+r)*z;
m = c + delta*k - w*l -r*k;
ms = m + d;
tr = (1-1/pi_bar)*ms;

%Compute aggregate output lost in monitoring
ya = y-c-e-g-I;
ya_y = ya/y;

%Compute information acquisition costs and monitoring costs
% output lost due to "melting cost" at bank
y_tau = tau*n*share_bank;
ytau_y = y_tau/y;

% output lost due to monitoring only
y_m = ya - y_tau;
ym_y = y_m/y;

%output net of agency costs
yp = y - ya;

%Compute the financial structure
lb = (1-tau)*sbp/(sc+0.0000000001);
aver_default = (expected_bank_monitoring+expected_cmf_monitoring)/(0.0000000001 + share_bank*(1-cond_dropout_if_banking)+share_cmf);
share_abstain_overall = share_abstain+share_bank*cond_dropout_if_banking;
aggr_debt_to_equity = (xi-1)*((1-tau)*sbp+sc);
aggr_debt_to_GDP = (xi-1)*((1-tau)*sbp+sc)*(n/(0.0000000001+y));

facts = [log(lb/5.48);
    log(aggr_debt_to_equity /.64);
    log(risk_premium_cmf/.0036); %.0143 annual
    log(risk_premium_bank/.0030); %.0119 annual
    log(cond_monitored_if_cmf/.0124); %.0496 annual
    log(expected_return/.023)%.093 annual
    ];

lng = length(facts);
A = eye(lng);

%compute mse
mse = facts'*A*facts

%print out deviations from target
exp_facts = exp(facts);


if (~DOING_ESTIMATION) & (~TABULATE_RESULTS) & (~DOING_SEARCH),
    
    gamma
    y
    l
    g_y;
    ya_y
    ytau_y
    ym_y
    I_y = I/y
    C_y = (c+e)/y
    c_y = c/y
    e_y = e/y
    z_K = z/K;
    n_y = n/y;
    
    
    share_abstain
    share_bank
    share_cmf
    cond_dropout_if_banking
    
    lb
    cond_monitored_if_cmf
    cond_monitored_if_banking
    aver_default
    share_abstain_overall
    risk_premium_bank
    risk_premium_cmf
    expected_return
    
    m_c = m/c
    
    results = [lb/5.48; %lb=5.6 in terms of outstanding amount, as in dfu1
    aggr_debt_to_equity /.64;
    risk_premium_cmf/.0036; %.0143 annual
    risk_premium_bank/.0030; %.0119 annual
    cond_monitored_if_cmf/.0124; %.0496 annual
    expected_return/.023;%.093 annual
    ]

    mse
    
    s_aggr1=s_aggr;
    sigma1_eps1=sigma_eps1;
    sigma1_eps2=sigma_eps2;
    sigma1_eps3=sigma_eps3;
    xi1=xi;
    tau1=tau;
    %state variables
    k1=k;
    z1=z;
    K1=k1+z1;
    I1=I;
    d1=d;
    ms1=ms;
    Rnom1=Rnom;
    %jump variables
    y1=y;
    e1 = e;
    n1= n;
    pi_bar1=pi_bar;
    ya1=ya;
    x1=x;
    lb1 = lb;
    dty1 = aggr_debt_to_GDP;
    rpb1=risk_premium_bank;
    rpc1 = risk_premium_cmf;
    defb1 = cond_monitored_if_banking;
    defc1 = cond_monitored_if_cmf;
    def1 = aver_default;
    sa1 = share_abstain;
    sb1 = share_bank;
    sbp1 = share_bank*(1-cond_dropout_if_banking);
    sc1 = share_cmf;
    m1 = m;
    tr1 = tr;
    h1 = l;
    w1 = w;
    rk1 = r;
    c1 = c;
    soa1 = share_abstain_overall;
    %other jump variables
    gamma1 = gamma;
    ya_y1= ya_y;
    ytau_y1=ytau_y;
    ym_y1 = ym_y;
    I_y1 = I_y;
    C_y1 = C_y;
    c_y1 = c_y;
    e_y1 = e_y;
    z_K1 = z_K;
    n_y1 = n/y;
    expected_return1 = expected_return;
    varkappa1 = varkappa;
    list_var_dyn1 = [k1 z1 d1 ms1 Rnom1 y1 e1 n1 s_aggr1 pi_bar1 ya1 x1 lb1 dty1 rpb1 rpc1 defb1 defc1 def1 sa1 sb1 sbp1 sc1 m1 tr1 h1 w1 rk1 c1 soa1 I1 0 0 0 0]' ;
    
    if DO_PERMANENT_SHOCK,
        
        save ss1_sv_ea  k1   z1   d1   ms1   Rnom1  s_aggr1  sigma1_eps1  sigma1_eps2  sigma1_eps3  xi1   tau1 varkappa1 gamma1 ya_y1 ytau_y1 ym_y1...
            I_y1  C_y1  c_y1  e_y1  z_K1  n_y1 expected_return1 y1 e1 n1 pi_bar1 ya1 x1 lb1 dty1 rpb1 rpc1 defb1 defc1 def1 sa1 sb1 sc1...
            m1 tr1 h1 w1 rk1 c1 soa1 I1 list_var_dyn1;
    end;

    
    if (~DO_PERMANENT_SHOCK),
        h = l;
        rk = r;
        rpb=risk_premium_bank;
        rpc = risk_premium_cmf;
        defb = cond_monitored_if_banking;
        defc = cond_monitored_if_cmf;
        def = aver_default;
        dty = aggr_debt_to_GDP;
        sa = share_abstain;
        sb = share_bank;
        sbp = share_bank*(1-cond_dropout_if_banking);
        sc = share_cmf;
        soa = share_abstain_overall;
        
        list_var_dyn = [k z d ms Rnom y e n  s_aggr pi_bar ya x lb dty rpb rpc defb defc def sa sb sbp sc m tr h w rk c soa I 0 0 0 0]' ;
        
        save ss_sv_ea  k z d ms Rnom s_aggr sigma_eps1 sigma_eps2 sigma_eps3  xi   tau gamma ya_y ytau_y ym_y...
            I_y C_y c_y e_y z_K n_y expected_return y e n pi_bar ya x lb dty rpb rpc defb defc def sa sb sc m...
            tr h w rk c soa I list_var_dyn;
    end;
    
end;


if (~DOING_ESTIMATION) && (TABULATE_RESULTS) &&  (~DOING_SEARCH),
    
    results_vector = [ ...
        y
        l
        ya/y
        I/y
        c/y
        e/y
        z/K
        K
        n
        y_tau/y
        y_m/y
        gamma
        share_abstain
        share_bank
        share_cmf
        cond_dropout_if_banking
        share_abstain_overall
        lb
        cond_monitored_if_cmf
        cond_monitored_if_banking
        aver_default
        risk_premium_bank
        risk_premium_cmf
        Rnom];
    
    
end;
